#!/usr/bin/ruby
# encoding: utf-8

class NetworkScreen	
	def initialize(extlayers)
		@layers = Array.new
		@nextscreen = "start"
		@networkcombo = Gtk::ComboBox.new
		@pwentry = Gtk::Entry.new
		### Some global variables for networks
		@networks = Array.new
		@radiowlan = nil
		@radioether = nil
		@radionone = nil
		
		nwscreen = create_connect_layer(extlayers)
		@layers[0] = nwscreen
		extlayers["networks"] = nwscreen
	end
	attr_reader :layers
	attr_accessor :nextscreen
	
	def create_connect_layer(extlayers)
		fixed = Gtk::Fixed.new
		tile = Gtk::Image.new("network.png")
		nwlabel = TileHelpers.create_label("<b>Netzwerkeinrichtung</b>\n\nEs wurde keine Netzwerkverbindung gefunden. Wie möchten Sie vorgehen?", 510)
		labelwlan = TileHelpers.create_label("WLAN verwenden", 250)
		labelether = TileHelpers.create_label("Ethernet verwenden (bitte schließen Sie jetzt ein Netzwerkkabel an)", 500)
		labelnone = TileHelpers.create_label("Ohne Netzwerkverbindung fortfahren", 500)
		namelab = TileHelpers.create_label("ESSID:", 100)
		passlab = TileHelpers.create_label("WPA-Schlüssel:", 100)
		continue = TileHelpers.create_label("Klicken Sie hier, um die Verbindung mit dem Netzwerk herzustellen.", 220)
		@pwentry.visibility = false
		@networkcombo.width_request = 330
		@networkcombo.height_request = 32
		@pwentry.width_request = 435
		reread = Gtk::Button.new("Neu einlesen")
		reread.height_request = 32
		reread.width_request = 100
		reread.signal_connect("clicked") { fill_wlan_combo } 
		@radiowlan = Gtk::RadioButton.new()
		@radioether = Gtk::RadioButton.new(@radiowlan)
		@radionone = Gtk::RadioButton.new(@radiowlan)
		forw  = Gtk::EventBox.new.add Gtk::Image.new("forward.png")
		fixed.put(tile, 0, 0)
		fixed.put(nwlabel, 130, 0)
		fixed.put(@radiowlan, 130, 60)
		fixed.put(labelwlan, 160, 65)
		fixed.put(namelab, 160, 93)
		fixed.put(passlab, 160, 130)
		fixed.put(@networkcombo, 265, 88)
		fixed.put(reread, 600, 88)
		fixed.put(@pwentry, 265, 125)
		fixed.put(@radioether, 130, 160)
		fixed.put(labelether, 160, 165)
		fixed.put(@radionone, 130, 190)
		fixed.put(labelnone, 160, 195)
		fixed.put(forw, 853, 282)
		fixed.put(continue, 605, 288)
		forw.signal_connect('button-release-event') { |x, y|
			if y.button == 1 
				if @radiowlan.active? 
					puts "Connect to WLAN..."
					connect_to_wlan
				elsif @radioether.active?
					puts "Connect to ethernet..."
					connect_to_ethernet
				end
				if test_connection || @radionone.active?
					extlayers.each { |k,v| v.hide_all }
					extlayers[@nextscreen].show_all
				elsif
					TileHelpers.error_dialog("Es konnte keine Netzwerkverbindung hergestellt werden!", "Fehler!")
				end
			end
		}
		return fixed
	end
	
	def connect_to_wlan
		return 1 if @networks.size < 1
		essid = @networks[@networkcombo.active] 
		key = @pwentry.text 
		
		# buttons.each { |b| b.sensitive = false }
		dwin = Gtk::Window.new
		dwin.set_default_size(400, 40)
		pgbar = Gtk::ProgressBar.new
		pgbar.width_request = 300
		dwin.add(pgbar)
		dwin.deletable = false
		dwin.show_all
		ccount = 0
		netnum = -1
		system("wicd-cli --wireless -S")
		IO.popen("wicd-cli --wireless -l") { |l|
			while l.gets 
				ltoks = $_.strip.split
				netnum = ltoks[0].to_i if ltoks[3] == essid
			end
		}
		if key == ""
			system("wicd-cli --wireless -n " + netnum.to_s + " -p encryption -s False")
		else
			system("wicd-cli --wireless -n " + netnum.to_s + " -p encryption -s True")
			system("wicd-cli --wireless -n " + netnum.to_s + " -p enctype -s wpa")
			system("wicd-cli --wireless -n " + netnum.to_s + " -p encryption_method -s wpa")
			system("wicd-cli --wireless -n " + netnum.to_s + " -p key -s " + key)
		end
		connecting_active = true
		connect_vte = Vte::Terminal.new
		connect_vte.signal_connect("child-exited") { connecting_active = false }
		connect_vte.fork_command("wicd-cli", [ "wicd-cli", "--wireless", "-n", netnum.to_s, "-c" ])
		while (Gtk.events_pending?)
			Gtk.main_iteration
		end
		while ( connecting_active == true && ccount < 1200 ) || ccount < 200
			pgbar.pulse
			pgbar.text = "Verbinde mit WLAN #{essid}"
			sleep 0.1
			ccount += 1
			while (Gtk.events_pending?)
				Gtk.main_iteration
			end
		end
		dwin.destroy
		# buttons.each { |b| b.sensitive = true }
	end
	
	def connect_to_ethernet
		dwin = Gtk::Window.new
		dwin.set_default_size(400, 40)
		pgbar = Gtk::ProgressBar.new
		pgbar.width_request = 300
		dwin.add(pgbar)
		dwin.deletable = false
		dwin.show_all
		ccount = 0
		connect_success = false
		while connect_success == false && ccount < 1200 
			pgbar.pulse
			pgbar.text = "Verbinde mit Ethernet"
			sleep 0.1
			ccount += 1
			connect_success = test_connection if ccount % 100 == 5 
			while (Gtk.events_pending?)
				Gtk.main_iteration
			end
			
		end
		dwin.destroy
	end
	
	def test_connection
		# Check the network by "digging" for www.computerbild.de
		# if this works, routing and name resolution should work
		return true if system("nslookup www.computerbild.de") # || system("nslookup www.computerbild.de 8.8.8.8")
		return false
	end
	
	def test_link
		link = false
		IO.popen("ip link") { |l| 
			while l.gets
				ltoks = $_.split
				if ltoks[1] =~ /eth/ 
					link = true unless ltoks[2] =~ /NO-CARRIER/
				end
			end
		} 
		return link 
	end
	
	def fill_wlan_combo
		100.downto(0) { |n|
			begin
				@networkcombo.remove_text(n)
			rescue
			end
		}
		@networks = Array.new
		get_networks.each { |k,v|
			v.each { |n| @networks.push(n) }
		}
		@networks.uniq.each { |n|
			@networkcombo.append_text n
		}
		@networkcombo.active = 0 unless @networks.size < 1 
	end
	
	def get_networks 
		# First find interfaces:
		interfaces = Array.new
		networks = Hash.new
		IO.popen("/usr/sbin/iwconfig 2>&1") { |l|
			while l.gets
				line = $_.strip
				if line =~ /ESSID/
					interfaces.push(line.split[0])
				end
			end
		}
		interfaces.each { |i|
			networks[i] = Array.new
			system("/sbin/ifconfig " + i + " up")
			IO.popen("/usr/sbin/iwlist " + i + " scan") { |l|
				while l.gets
					line = $_.strip
					if line =~ /^ESSID\:\"(.*?)\"/
						networks[i].push($1)
					end
				end
			}
		}
		return networks
	end
	
	def get_addresses
		addresses = Array.new
		IO.popen("ip -f inet address") { |l|
			while l.gets
				line = $_.strip
				if line =~ /^inet/
					ltoks = line.split 
					addtoks = ltoks[1].split("/")
					addresses.push(addtoks[0]) unless addtoks[0] =~ /^127.0/ 
				end
			end
		}
		return addresses 
	end
	
end